"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_1 = require("servie/dist/node");
const common_1 = require("servie/dist/common");
const pathToRegexp = require("path-to-regexp");
const servie_url_1 = require("servie-url");
const popsicle_transport_http_1 = require("popsicle-transport-http");
/**
 * Create a simple service to proxy requests.
 */
class Service {
    constructor(config) {
        this.re = config.path
            ? pathToRegexp(config.path, [], { strict: true })
            : undefined;
        this.methods = config.methods
            ? new Set(config.methods.map(x => x.toLowerCase()))
            : undefined;
        this.url = config.url;
    }
    isMethod(method) {
        return this.methods ? this.methods.has(method.toLowerCase()) : true;
    }
    isPath(path) {
        return this.re ? this.re.test(path) : true;
    }
    matches(req) {
        return (this.isMethod(req.method) && this.isPath(servie_url_1.getURL(req).pathname || "/"));
    }
}
exports.Service = Service;
/**
 * Proxy any number of paths to services.
 */
function proxy(configs) {
    const services = configs.map(config => new Service(config));
    // Forward HTTP requests raw without any pre-processing.
    const send = popsicle_transport_http_1.transport();
    return (req, next) => __awaiter(this, void 0, void 0, function* () {
        for (const service of services) {
            if (service.matches(req)) {
                const proxyReq = new node_1.Request(`${service.url}${req.url}`, {
                    method: req.method,
                    headers: req.headers,
                    signal: req.signal,
                    omitDefaultHeaders: true,
                    body: common_1.useRawBody(req),
                    trailer: req.trailer
                });
                return send(proxyReq, () => {
                    return Promise.reject(new TypeError("Unhandled proxy request"));
                });
            }
        }
        return next(); // 404.
    });
}
exports.proxy = proxy;
//# sourceMappingURL=index.js.map